@@ -8,6 +8,7 @@ from .prompt import (
     TokenControl,
     Tokens,
 )
+from .prompt_template import PromptTemplate
 from .aleph_alpha_client import (
     POOLING_OPTIONS,
     AsyncClient,
@@ -20,6 +21,8 @@ from .document import Document
 from .embedding import (
     EmbeddingRequest,
     EmbeddingResponse,
+    BatchSemanticEmbeddingRequest,
+    BatchSemanticEmbeddingResponse,
     SemanticEmbeddingRequest,
     SemanticEmbeddingResponse,
     SemanticRepresentation,
@@ -33,6 +36,7 @@ from .explanation import (
     ExplanationResponse,
     ImagePromptItemExplanation,
     ImageScore,
+    PromptGranularity,
     TargetGranularity,
     TargetPromptItemExplanation,
     TargetScore,
@@ -42,7 +46,6 @@ from .explanation import (
     TokenScore,
 )
 from .qa import QaRequest, QaResponse
-from .search import SearchRequest, SearchResponse, SearchResult
 from .summarization import SummarizationRequest, SummarizationResponse
 from .tokenization import TokenizationRequest, TokenizationResponse
 from .utils import load_base64_from_file, load_base64_from_url
@@ -50,6 +53,8 @@ from .version import __version__
 
 __all__ = [
     "AsyncClient",
+    "BatchSemanticEmbeddingRequest",
+    "BatchSemanticEmbeddingResponse",
     "Client",
     "CompletionRequest",
     "CompletionResponse",
@@ -72,12 +77,11 @@ __all__ = [
     "ImageScore",
     "POOLING_OPTIONS",
     "Prompt",
+    "PromptTemplate",
+    "PromptGranularity",
     "QaRequest",
     "QaResponse",
     "QuotaError",
-    "SearchRequest",
-    "SearchResponse",
-    "SearchResult",
     "SemanticEmbeddingRequest",
     "SemanticEmbeddingResponse",
     "SemanticRepresentation",
