@@ -1,13 +1,26 @@
 from tokenizers import Tokenizer  # type: ignore
 from types import TracebackType
-from typing import Any, List, Mapping, Optional, Dict, Type, Union
+from typing import (
+    Any,
+    List,
+    Mapping,
+    Optional,
+    Dict,
+    Sequence,
+    Tuple,
+    Type,
+    Union,
+    Iterator,
+)
 import aiohttp
+import asyncio
 from aiohttp_retry import RetryClient, ExponentialRetry
 import requests
 from requests import Response
 from requests.adapters import HTTPAdapter
 from requests.structures import CaseInsensitiveDict
 from urllib3.util.retry import Retry
+from tqdm.asyncio import tqdm
 
 import aleph_alpha_client
 from aleph_alpha_client.explanation import (
@@ -16,7 +29,6 @@ from aleph_alpha_client.explanation import (
     ExplanationRequest,
     ExplanationResponse,
 )
-from aleph_alpha_client.prompt import _to_json, _to_serializable_prompt
 from aleph_alpha_client.summarization import SummarizationRequest, SummarizationResponse
 from aleph_alpha_client.qa import QaRequest, QaResponse
 from aleph_alpha_client.completion import CompletionRequest, CompletionResponse
@@ -27,12 +39,14 @@ from aleph_alpha_client.detokenization import (
     DetokenizationResponse,
 )
 from aleph_alpha_client.embedding import (
+    BatchSemanticEmbeddingRequest,
+    BatchSemanticEmbeddingResponse,
     EmbeddingRequest,
     EmbeddingResponse,
+    EmbeddingVector,
     SemanticEmbeddingRequest,
     SemanticEmbeddingResponse,
 )
-from aleph_alpha_client.search import SearchRequest, SearchResponse
 
 POOLING_OPTIONS = ["mean", "max", "last_token", "abs_max"]
 RETRY_STATUS_CODES = frozenset({408, 429, 500, 502, 503, 504})
@@ -72,11 +86,11 @@ AnyRequest = Union[
     TokenizationRequest,
     DetokenizationRequest,
     SemanticEmbeddingRequest,
+    BatchSemanticEmbeddingRequest,
     QaRequest,
     SummarizationRequest,
     ExplanationRequest,
     ExplanationRequest,
-    SearchRequest,
 ]
 
 
@@ -132,7 +146,6 @@ class Client:
         total_retries: int = 8,
         nice: bool = False,
     ) -> None:
-
         if host[-1] != "/":
             host += "/"
         self.host = host
@@ -176,7 +189,6 @@ class Client:
         request: AnyRequest,
         model: Optional[str] = None,
     ) -> Dict[str, Any]:
-
         json_body = self._build_json_body(request, model)
 
         query_params = self._build_query_parameters()
@@ -343,7 +355,7 @@ class Client:
 
         Parameters:
             request (SemanticEmbeddingRequest, required):
-                Parameters for the requested semnatic embedding.
+                Parameters for the requested semantic embedding.
 
             model (string, required):
                 Name of model to use. A model name refers to a model architecture (number of parameters among others).
@@ -384,6 +396,54 @@ class Client:
         )
         return SemanticEmbeddingResponse.from_json(response)
 
+    def batch_semantic_embed(
+        self,
+        request: BatchSemanticEmbeddingRequest,
+        model: Optional[str] = None,
+    ) -> BatchSemanticEmbeddingResponse:
+        """Embeds a sequence of texts or images and returns vectors in the same order as they
+        were provided. If more than 100 prompts are provided then this method will chunk them
+        into batches of 100 prompts that will be sent to the API.
+
+        Parameters:
+            request (BatchSemanticEmbeddingRequest, required):
+                Parameters for the requested semantic embeddings.
+
+            model (string, optional, default None):
+                Name of model to use. A model name refers to a model architecture (number of parameters among others).
+                Always the latest version of model is used.
+
+        Examples:
+            >>> # function for symmetric embedding
+            >>> def embed_symmetric(texts: Sequence[str]):
+                    # Create an embeddingrequest with the type set to symmetric
+                    request = BatchSemanticEmbeddingRequest(
+                        prompts=[Prompt.from_text(text) for text in texts],
+                        representation=SemanticRepresentation.Symmetric
+                    )
+                    # create the embedding
+                    result = client.batch_semantic_embed(request, model=model_name)
+                    return result.embedding
+        """
+
+        responses: List[EmbeddingVector] = []
+        model_version = ""
+        # The API currently only supports batch semantic embedding requests with up to 100
+        # prompts per batch. As a convenience for users, this function chunks larger requests.
+        for batch_request in _generate_semantic_embedding_batches(request):
+            raw_response = self._post_request(
+                "batch_semantic_embed",
+                batch_request,
+                model,
+            )
+            response = BatchSemanticEmbeddingResponse.from_json(raw_response)
+            model_version = response.model_version
+            responses.extend(response.embeddings)
+
+        return BatchSemanticEmbeddingResponse._from_model_version_and_embeddings(
+            model_version, responses
+        )
+
     def evaluate(
         self,
         request: EvaluationRequest,
@@ -395,7 +455,7 @@ class Client:
             request (EvaluationRequest, required):
                 Parameters for the requested evaluation.
 
-            model (string, optional, default None):
+            model (string, required):
                 Name of model to use. A model name refers to a model architecture (number of parameters among others).
                 Always the latest version of model is used.
 
@@ -451,11 +511,29 @@ class Client:
         )
         return SummarizationResponse.from_json(response)
 
-    def _explain(
+    def explain(
         self,
         request: ExplanationRequest,
         model: str,
     ) -> ExplanationResponse:
+        """Better understand the source of a completion, specifically on how much each section of a
+        prompt impacts each token of the completion.
+
+        Parameters:
+            request (ExplanationRequest, required):
+                Parameters for the requested explanation.
+
+            model (string, required):
+                Name of model to use. A model name refers to a model architecture (number of parameters among others).
+                Always the latest version of model is used.
+
+        Examples:
+            >>> request = ExplanationRequest(
+                    prompt=Prompt.from_text("Andreas likes"),
+                    target=" pizza."
+                )
+            >>> response = client.explain(request, model="luminous-extended")
+        """
         response = self._post_request(
             "explain",
             request,
@@ -463,16 +541,6 @@ class Client:
         )
         return ExplanationResponse.from_json(response)
 
-    def _search(
-        self,
-        request: SearchRequest,
-    ) -> SearchResponse:
-        """
-        For details see https://www.aleph-alpha.com/luminous-explore-a-model-for-world-class-semantic-representation
-        """
-        response = self._post_request("search", request, None)
-        return SearchResponse.from_json(response)
-
     def tokenizer(self, model: str) -> Tokenizer:
         """Returns a Tokenizer instance with the settings that were used to train the model.
 
@@ -543,6 +611,7 @@ class AsyncClient:
 
         retry_options = ExponentialRetry(
             attempts=total_retries + 1,
+            exceptions={aiohttp.ClientConnectionError},
             start_timeout=0.25,
             statuses=set(RETRY_STATUS_CODES),
         )
@@ -614,7 +683,6 @@ class AsyncClient:
         request: AnyRequest,
         model: Optional[str] = None,
     ) -> Dict[str, Any]:
-
         json_body = self._build_json_body(request, model)
 
         query_params = self._build_query_parameters()
@@ -700,7 +768,7 @@ class AsyncClient:
             request (TokenizationRequest, required):
                 Parameters for the requested tokenization.
 
-            model (string, optional, default None):
+            model (string, required):
                 Name of model to use. A model name refers to a model architecture (number of parameters among others).
                 Always the latest version of model is used.
 
@@ -726,7 +794,7 @@ class AsyncClient:
             request (DetokenizationRequest, required):
                 Parameters for the requested detokenization.
 
-            model (string, optional, default None):
+            model (string, required):
                 Name of model to use. A model name refers to a model architecture (number of parameters among others).
                 Always the latest version of model is used.
 
@@ -752,7 +820,7 @@ class AsyncClient:
             request (EmbeddingRequest, required):
                 Parameters for the requested embedding.
 
-            model (string, optional, default None):
+            model (string, required):
                 Name of model to use. A model name refers to a model architecture (number of parameters among others).
                 Always the latest version of model is used.
 
@@ -777,9 +845,9 @@ class AsyncClient:
 
         Parameters:
             request (SemanticEmbeddingRequest, required):
-                Parameters for the requested semnatic embedding.
+                Parameters for the requested semantic embedding.
 
-            model (string, optional, default None):
+            model (string, required):
                 Name of model to use. A model name refers to a model architecture (number of parameters among others).
                 Always the latest version of model is used.
 
@@ -817,6 +885,73 @@ class AsyncClient:
         )
         return SemanticEmbeddingResponse.from_json(response)
 
+    async def batch_semantic_embed(
+        self,
+        request: BatchSemanticEmbeddingRequest,
+        model: Optional[str] = None,
+        num_concurrent_requests: int = 1,
+        batch_size: int = 100,
+        progress_bar: bool = False,
+    ) -> BatchSemanticEmbeddingResponse:
+        """Embeds a sequence of texts or images and returns vectors in the same order as they
+        were provided. If more than `batch_size` prompts are provided then this method will chunk them
+        into batches of up to `batch_size` prompts that will be sent to the API.
+
+        Parameters:
+            request (BatchSemanticEmbeddingRequest, required):
+                Parameters for the requested semantic embeddings.
+
+            model (string, optional, default None):
+                Name of model to use. A model name refers to a model architecture (number of parameters among others).
+                Always the latest version of model is used.
+
+            num_concurrent_requests (int, optional, default 1):
+                Maximum number of concurrent requests to send to the API.
+
+            batch_size (int, optional, default 100):
+                Number of prompts per batch sent to the API. This value must be between 1 and 100 (inclusive).
+
+            progress_bar (bool, optional, default False):
+                Whether to show a progress bar using tqdm.
+
+        Examples:
+            >>> # function for symmetric embedding
+            >>> def embed_symmetric(texts: Sequence[str]):
+                    # Create an embeddingrequest with the type set to symmetric
+                    request = BatchSemanticEmbeddingRequest(
+                        prompts=[Prompt.from_text(text) for text in texts],
+                        representation=SemanticRepresentation.Symmetric
+                    )
+                    # create the embedding
+                    result = client.batch_semantic_embed(request, model=model_name)
+                    return result.embedding
+        """
+        if batch_size < 1 or batch_size > 100:
+            raise ValueError(
+                "`batch_semantic_embed` must be called with a `batch_size` between 1 and 100 (inclusive)"
+            )
+
+        responses: List[EmbeddingVector] = []
+        model_version = ""
+
+        # The API currently only supports batch semantic embedding requests with up to 100
+        # prompts per batch. As a convenience for users, this function chunks larger requests.
+        results = await self._gather_with_concurrency(
+            "batch_semantic_embed",
+            model,
+            num_concurrent_requests,
+            _generate_semantic_embedding_batches(request, batch_size),
+            progress_bar,
+        )
+        for result in results:
+            resp = BatchSemanticEmbeddingResponse.from_json(result)
+            model_version = resp.model_version
+            responses.extend(resp.embeddings)
+
+        return BatchSemanticEmbeddingResponse._from_model_version_and_embeddings(
+            model_version, responses
+        )
+
     async def evaluate(
         self,
         request: EvaluationRequest,
@@ -828,7 +963,7 @@ class AsyncClient:
             request (EvaluationRequest, required):
                 Parameters for the requested evaluation.
 
-            model (string, optional, default None):
+            model (string, required):
                 Name of model to use. A model name refers to a model architecture (number of parameters among others).
                 Always the latest version of model is used.
 
@@ -871,11 +1006,6 @@ class AsyncClient:
         Parameters:
             request (SummarizationRequest, required):
                 Parameters for the requested summarization.
-
-            model (string, optional, default None):
-                Name of model to use. A model name refers to a model architecture (number of parameters among others).
-                Always the latest version of model is used.
-
         Examples:
             >>> request = SummarizationRequest(
                     document=Document.from_text("Andreas likes pizza."),
@@ -888,11 +1018,29 @@ class AsyncClient:
         )
         return SummarizationResponse.from_json(response)
 
-    async def _explain(
+    async def explain(
         self,
         request: ExplanationRequest,
         model: str,
     ) -> ExplanationResponse:
+        """Better understand the source of a completion, specifically on how much each section of a
+        prompt impacts each token of the completion.
+
+        Parameters:
+            request (ExplanationRequest, required):
+                Parameters for the requested explanation.
+
+            model (string, required):
+                Name of model to use. A model name refers to a model architecture (number of parameters among others).
+                Always the latest version of model is used.
+
+        Examples:
+            >>> request = ExplanationRequest(
+                    prompt=Prompt.from_text("Andreas likes"),
+                    target=" pizza."
+                )
+            >>> response = await client.explain(request, model="luminous-extended")
+        """
         response = await self._post_request(
             "explain",
             request,
@@ -900,16 +1048,6 @@ class AsyncClient:
         )
         return ExplanationResponse.from_json(response)
 
-    async def _search(
-        self,
-        request: SearchRequest,
-    ) -> SearchResponse:
-        """
-        For details see https://www.aleph-alpha.com/luminous-explore-a-model-for-world-class-semantic-representation
-        """
-        response = await self._post_request("search", request, None)
-        return SearchResponse.from_json(response)
-
     async def tokenizer(self, model: str) -> Tokenizer:
         """Returns a Tokenizer instance with the settings that were used to train the model.
 
@@ -919,3 +1057,43 @@ class AsyncClient:
         """
         response = await self._get_request_text(f"models/{model}/tokenizer")
         return Tokenizer.from_str(response)
+
+    # Based on: https://docs.aleph-alpha.com/changelog/2022/11/14/async-python-client/
+    async def _gather_with_concurrency(
+        self,
+        endpoint: str,
+        model: Optional[str],
+        n: int,
+        requests: Sequence[BatchSemanticEmbeddingRequest],
+        progress_bar: bool,
+    ) -> List[Dict[str, Any]]:
+        semaphore = asyncio.Semaphore(n)
+
+        async def sem_task(request: BatchSemanticEmbeddingRequest):
+            async with semaphore:
+                return await self._post_request(endpoint, request, model)
+
+        # asyncio.gather preserves order of awaitables in result list
+        if progress_bar:
+            return await tqdm.gather(*(sem_task(request) for request in requests))
+        else:
+            return await asyncio.gather(*(sem_task(request) for request in requests))
+
+
+def _generate_semantic_embedding_batches(
+    request: BatchSemanticEmbeddingRequest, batch_size: int = 100
+) -> List[BatchSemanticEmbeddingRequest]:
+    requests = []
+    for batch_index in range(0, len(request.prompts), batch_size):
+        batch = request.prompts[batch_index : batch_index + batch_size]
+        requests.append(
+            BatchSemanticEmbeddingRequest(
+                prompts=batch,
+                representation=request.representation,
+                compress_to_size=request.compress_to_size,
+                normalize=request.normalize,
+                contextual_control_threshold=request.contextual_control_threshold,
+                control_log_additive=request.control_log_additive,
+            )
+        )
+    return requests
